<?php
/**
 * PercorreCNAB.
 *
 * @package Logic
 *
 * @author  thalesvalias - 26/07/2012
 */
class PercorreCNAB{

    //private $_parametros;
    private $_formulario;
    private $_retrocesso;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_idGeraMeioMagnetico;
    private $_idProcessaVinculo;
    private $_nrEstruturaResp;
    private $_filePath;
    private $_tpInscResp;
    private $_nrEvento;
    private $_nrTipoMovimento;
    private $_cdBanco;
    private $_nrGrupoCnab;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_calculoFolha;
    private $_nrCalculoFolha;
    private $_cnabMagnetico;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_vinculom;
    private $_pessoa;
    private $_dtPagamento;
    private $_estruturacao;
    private $_nomeArquivo;
    private $_cnab;
    private $_msgSucessoPersisteMagnetico;
    private $_msgSucessoGeraMagnetico;
    private $_contador;
    private $_idUtilDtPgtoItem;
    private $_tpVinculo;

    const PREFIXO_NOME_ARQUIVO_CNAB = 'cnab';
    const EXTENSAO_ARQUIVO = '.txt';

    /**
     * Construtor.
     *
     * @param string     $tpProcessamento Tipo de processamento (MENSAL).
     * @param integer    $nrFormulario    Numero do formulario.
     * @param Parametros $parametro       Instancia de Parametros.
     *
     * @author thalesvalias - 26/07/2012
     */
    public function __construct($tpProcessamento, $nrFormulario, Parametros $parametro, $tpvinculo = null){

        $this->_nrOrg = \FPA::getNrorgTrabalho();
        $this->_nrOrgPadrao = \FPA::getNrorgPadrao();
        $this->_nrEstruturaTrab = \FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = new TkDateTime(\FPA::getDtCompetencia());
        $this->_nrTipoModalidadeCalculo = \FPA::getNrModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = \FPA::getOcorrenciaCalculo();
        $this->_vinculom = new GpevinculomEntity();
        $this->_tipoHierarquia = new TipohierarqorgEntity();
        $this->_hierarquiaOrg = new HierarquiaorgEntity();
        $this->_calculoFolha = new FpacalculofolhaEntity();
        $this->_cnabMagnetico = new FpaCnabmagnetEntity();
        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_inicializaVariaveis($nrFormulario);
        $this->_tpVinculo = $tpvinculo;

        $tpProcessamento = strtoupper($tpProcessamento);
        switch ($tpProcessamento){
        case Parametros::PROC_COMPETENCIA_MENSAL :
             $this->_retrocesso = false;
             break;
        case Parametros::PROC_RETROCESSO :
            $this->_retrocesso = true;
            break;
        default:
            tk_message('Tipo de processo '.$tpProcessamento.' inválido', 'warning');
        }

        $this->_cnab = new CNAB(
            $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_cnabMagnetico, $this->_formulario, $this->_nrCalculoFolha
        );
    }


    public function _inicializaVariaveis($nrFormulario) {
        $this->_buscaDadosFormulario($nrFormulario);

        $calculoFolha = $this->_calculoFolha->retornaCalculoFolha(
            $this->_nrOrg, $this->_nrTipoMovimento,$this->_mesCompetencia->toString(TkDateTime::DAY.'/'.TkDateTime::MONTH.'/'.TkDateTime::YEAR),$this->_nrTipoModalidadeCalculo, $this->_nrOcorrenciaCalculo);
        $this->_nrCalculoFolha = end($calculoFolha)->getNrcalculofolha();

        $this->_msgSucessoPersisteMagnetico = null;
        $this->_msgSucessoGeraMagnetico = null;
    }

    /**
     * Percorre os vinculos para geracao dos registros.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     *
     * @author thalesvalias - 26/07/2012
     */
    public function percorre(){
        if ($this->_retrocesso === false) {

                $this->_geraCNAB();

                if ($this->_msgSucessoPersisteMagnetico !== null) {
                    tk_message($this->_msgSucessoPersisteMagnetico,'success');
                }
                if ($this->_msgSucessoGeraMagnetico !== null) {
                    tk_message($this->_msgSucessoGeraMagnetico,'success');
                }

        } else {

                $this->_cnab->retroceder();
                tk_message('Retrocedido', 'success');

        }
    }


    /**
     * Gera os registros do arquivo CNAB.
     *
     * @throws Exception Propaga Excecao.
     *
     * @return void
     *
     * @author thalesvalias - 26/07/2012
     */
    private function _geraCNAB(){

        // Percorre os vinculos gerando os registro do CNAB.        
        if ($this->_idProcessaVinculo === true) {

            try{
                // Inicializa Lista do Magnetico.
                $this->_cnabMagnetico->inicializaListaRegistros(
                    $this->_mesCompetencia, RegistroCNAB::LISTA_TIPO_REGISTROS_UNICO, $this->_nrOrg, $this->_cdBanco, $this->_nrTipoMovimento
                );

                // felipesantos - 08/04/2015 : verificação se utiliza data de pagamento
                $dtPagamento = $this->_idUtilDtPgtoItem==='S' ? $this->_dtPagamento : null;

//                 Retorna todos os vinculos.
                $vinculos = $this->_vinculom->getVinculosAtivosParaEvento(
                    $this->_nrOrg, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_nrEvento,
                    $this->_nrTipoMovimento, $this->_dsOrdem, $dtPagamento, $this->_tpVinculo
                );

            }
            catch(Exception $e){
                tk_warning("Erro ao inicializar preparação do CNAB");
                throw $e;
            }

            // Percorre os vinculos gerando cnab.
            foreach ($vinculos as $vinculo) {
                $this->_cnab->preparaCNAB($vinculo);
            }

            // So persiste se a lista nao estiver vazia.
            $qtdRegistros = count($this->_cnabMagnetico->getItems());

            if ($qtdRegistros >/*==*/ 0) { //ORIGINAL! 14/03/14 pck
                $persisteCnab = $this->_cnabMagnetico->persist();
                if ($persisteCnab === true) {
                    $this->_msgSucessoPersisteMagnetico = "CNAB gravado com sucesso!";
                }
            }
        }else{
            tk_message('Selecione para processar o vinculo','info');
        }

        // Gera o magnetico.
        if ($this->_idGeraMeioMagnetico === true) {
            try{
                $this->_geraMagnetico();
            }
            catch(Exception $e){
                tk_warning("Erro ao gerar magnético.".$e->getMessage());
            }
        }else{
            tk_message('Selecione para gerar o magnetico','info');
        }
    }


    /**
     * Gera o magnetico: grava todos os registros na base e gera o arquivo CNAB.
     *
     * @return void
     *
     * @author thalesvalias - 07/08/2012
     */
    private function _geraMagnetico(){

        // Busca da base os resgistros daquela competencia, banco e tipo de movimento ordenados por inscricao e tipo de registro.
        $resultado = $this->_cnabMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg, $this->_cdBanco, $this->_nrTipoMovimento);
        if (count($resultado) > 0) {
        //if (count($resultado) == 0) {
            // Define o nome do arquivo.
            $this->_constroiNomeArquivo();

            // Monta o diretorio completo, incluindo o nome do arquivo a ser gerado.
            //$caminhoArquivo = $this->_filePath."\\".$this->_nomeArquivo;

            // Criar o arquivo.
            //$arquivo = new TkFile($caminhoArquivo, true);

            //Chama método para gerar o arquivo, passando os parâmetros: nome da pasta, nome do arquivo e extensão arquivo
            $nomePasta = "CAGED";
            $nomeArquivo = $this->_nomeArquivo;
            $extensao = self::EXTENSAO_ARQUIVO;
            $arquivo = \FPA::geraArquivo($nomePasta, $nomeArquivo, $extensao);

            $this->_contador = 1;
            // Loop no resultado. Armazena os registros, modifica as linhas necessarias e adiciona quebra de linha.
            foreach ($resultado as $item) {

                // O resultado dos registros ja vem ordenado. Modifica as linhas necessarias (consolidacao e sequencia).
                $linhaRegistro = $this->_modificaLinhas($item);

                // Se for registro 5, seta o valor total do pagamento para persistir no magnetico.
                if ($item->getIdtiporegistro() === Registro5CNABSantander::REGISTRO_5) {
                    $valorTotalPagamento = $this->_cnab->getValorTotalPagamento();
                    $item->setValor($valorTotalPagamento);
                }

                $item->persist();
                $arquivo->add($linhaRegistro);

                // Nao adiciona quebra de linha apos inclusao da ultima linha.
                if (end($resultado) !== $item) {
                    $arquivo->add(RegistroCNAB::QUEBRA_LINHA_CR_LF);
                }
            }

            $geraArquivo = $arquivo->save(false, null, 0777);

            if (empty($geraArquivo) === false) {
                $arquivo->download(substr($_SERVER['REQUEST_URI'], 0, strlen($_SERVER['REQUEST_URI']) - 12)."/".\FPA::getNrorgTrabalho()."/".$nomePasta."/".$nomeArquivo);
                $this->_msgSucessoGeraMagnetico =  "Arquivo gerado com sucesso!";
            }
        }else{
            tk_message('Não existe registro processado para o vinculo '.$this->_vinculom->getNrvinculom(),'warning');
            //tk_message('Restruturar esta parte: Não existe registro processado para o vinculo '.$this->_vinculom->getNrvinculom(),'warning');
        }
    }


    /**
     * Modifica a linha informada de acordo com as necessidades (squencia, consolidacao) e a retorna.
     *
     * @param FpaCnabmagnetEntity $itemCnabMagnetico instancia de FpaCnabmagnetEntity.
     *
     * @return string (linha modificada do registro)
     *
     * @author thalesvalias - 07/08/2012
     */
    private function _modificaLinhas(FpaCnabmagnetEntity $itemCnabMagnetico) {
        // Caso o registro seja do tipo 3, modifica sua sequencia.
        $linhaRegistro = $itemCnabMagnetico->getDslinharegistro();
        $tipoRegistro = $itemCnabMagnetico->getIdtiporegistro();

        if ($tipoRegistro === RegistroCNAB::REGISTRO_3) {
            $linhaRegistro = $this->_cnab->modificaSequencia($linhaRegistro, $this->_contador);
            $itemCnabMagnetico->setDslinharegistro($linhaRegistro);
            $this->_contador++;
        } else {
            $linhaRegistro = $this->_cnab->modificaLinhasConsolidacao($tipoRegistro, $linhaRegistro);
            $itemCnabMagnetico->setDslinharegistro($linhaRegistro);
        }

        return $linhaRegistro;
    }


    /**
     * Constroi o nome do arquivo magnetico do cnab.
     *
     * @return void
     *
     * @author thalesvalias - 10/08/2012
     */
    private function _constroiNomeArquivo() {
        $ultimoSequencial = $this->_cnabMagnetico->retornaUltimoSequencialRegistro0($this->_mesCompetencia,
            $this->_nrOrg, $this->_cdBanco);

        if (empty($ultimoSequencial) === false) {
            $sequencia = $ultimoSequencial;
            $sequencia++;
        } else {
            $sequencia = FPARegistro::CARACTER_ONZE;
        }

        $this->_nomeArquivo = 'PG'.(string)$sequencia.$this->_mesCompetencia->toString(TkDateTime::MONTH).self::EXTENSAO_ARQUIVO;

//        $anoCompetencia = $this->_mesCompetencia->toString(TkDateTime::YEAR);
//        $mesCompetencia = $this->_mesCompetencia->toString(TkDateTime::MONTH);
//        // Verifica se o arquivo a ser gerado sera de acerto ou nao. O tratamento do nome varia de acordo com o tipo de atualizacao.
//        $this->_nomeArquivo = self::PREFIXO_NOME_ARQUIVO_CNAB.$mesCompetencia.$anoCompetencia.FPARegistro::CARACTER_UNDERLINE
//            .$this->_cdBanco.FPARegistro::CARACTER_UNDERLINE.$this->_nrTipoMovimento.self::EXTENSAO_ARQUIVO;
    }


    /**
     * Busca dados do formulario.
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @return void
     *
     * @author thalesvalias - 26/07/2012
     */
    private function _buscaDadosFormulario($nrFormulario){
        $formulario = new FpaFormcnabEntity();
        $this->_formulario = $formulario->retornaDadosFormularioCNAB($this->_nrOrg, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_dsInicio = $this->_formulario->getDsinicio();
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_nrTpHierarquiaOrg = $this->_formulario->getNrhierarqorg();
            $this->_idProcessaVinculo = $this->_formulario->getIdprocvinculo();
            $this->_idGeraMeioMagnetico = $this->_formulario->getIdgerameiomag();
            $this->_filePath = $this->_formulario->getDscaminhoarquivo();
            $this->_nrEstruturaResp = $this->_formulario->getNrestrutrespons();
            $this->_tpInscResp = $this->_formulario->getIdtpinscrespons();
            $this->_nrEvento = $this->_formulario->getNreventom();
            $this->_nrTipoMovimento = $this->_formulario->getNrtipomoviment();
            $this->_cdBanco = $this->_formulario->getCdbanco();
            $this->_nrGrupoCnab = $this->_formulario->getNrgrupocnab();
            $this->_dtPagamento = $this->_formulario->getDtpagamento();
            $this->_idUtilDtPgtoItem = $this->_formulario->getIdutildtpgtoitem();
        }
    }


}